package org.postgresql.net;

import io.netty.buffer.PooledByteBufAllocator;
import io.netty.handler.codec.http.DefaultFullHttpResponse;
import io.netty.handler.codec.http.FullHttpRequest;
import io.netty.handler.codec.http.FullHttpResponse;
import io.netty.handler.codec.http.HttpResponseStatus;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import net.piraku.concurrent.FutureResource;
import net.piraku.core.CoSync;
import net.piraku.core.PirakuResources;
import net.piraku.sql.ConnectionResource;
import net.piraku.sql.ResultSetResource;
import net.piraku.sql.StatementResource;

import com.fasterxml.jackson.databind.ObjectMapper;

public class DbQueryHandler {
	ConPool pool;

	public DbQueryHandler() throws Exception {
		pool = ConPool.getPool();
	}
	
	ConnectionResource getNext() {
		return pool.getNext();
	}
	
	public static class Message {
		private int id;
		private int randomNumber;
		
		public Message(int id, int num) {
			this.id = id;
			this.randomNumber = num;
		}
		
		public int getId() {
			return id;
		}
		public int getRandomNumber() {
			return randomNumber;
		}
		
		@Override
		public String toString() {
			return "id="+id+", num="+randomNumber;
		}
		
	}
	
	ObjectMapper mapper = new ObjectMapper();
	private Random random = new Random();
	
	@CoSync
	public FullHttpResponse service(FullHttpRequest req) throws Exception {
//		UUID num = UUID.randomUUID();
//		System.out.println("starting "+num);
		ConnectionResource con = getNext();
		
		int queries = 1;
		
		String uri = req.uri();
		int idx = uri.indexOf("queries=");
		if (-1!= idx) {
			uri = uri.substring(idx);
			if (uri.contains("&")) {
				idx = uri.indexOf("&");
				uri = uri.substring(0, idx);
			}
			
			uri = uri.substring(8);
			
			queries = Integer.parseInt(uri);
			if (queries > 500)
				queries = 500;
			if (queries < 1)
				queries = 1;
		}
		
		List<StatementResource > statements = new ArrayList<StatementResource >();
		List<FutureResource<ResultSetResource>> results = new ArrayList<FutureResource<ResultSetResource>>();
		for (int i=0; i<queries; i++) {
			StatementResource stmt = con.createStatement();
//			stmt.setInt(1, random.nextInt(10000));
			FutureResource<ResultSetResource> rsFuture = stmt.executeQueryAsync("select id, randomNumber from world where id = "+random.nextInt(10000));
			statements.add(stmt);
			results.add(rsFuture);
		}
		
		ArrayList<Message> messages = new ArrayList<DbQueryHandler.Message>();
		for (int i=0; i<queries; i++) {
			
			StatementResource stmt = statements.get(i);
			FutureResource<ResultSetResource> rsFuture = results.get(i);
//			System.out.println("Probing: "+rsFuture);
			PirakuResources.await(rsFuture);
			ResultSetResource rs = rsFuture.get();
			Message msg = null;
			
			if (rs.next())
				msg = new Message(rs.getInt(1), rs.getInt(2));
			
			messages.add(msg);
			rs.close();
			stmt.close();
		}
		
		
//		System.out.println(""+System.currentTimeMillis()+" -> "+ msg);
		
		
		byte[] hello = mapper.writeValueAsBytes(messages);
		DefaultFullHttpResponse res = new DefaultFullHttpResponse(req.protocolVersion(), HttpResponseStatus.OK, PooledByteBufAllocator.DEFAULT.buffer(hello.length));
		res.headers().add("Content-Type", "application/json; charset=UTF-8");
		res.headers().add("Server", "Example");
		res.headers().add("Date", new Date());
		res.headers().add("Content-Length", hello.length);
		if (req.headers().contains("Connection"))
			res.headers().add("Connection", req.headers().get("Connection"));
		else
			res.headers().add("Connection", "Close");
		
		res.content().writeBytes(hello);
		
//		System.out.println("done "+num);
		return res;
	}
	
	

}
