package org.postgresql.net;

import io.netty.buffer.PooledByteBufAllocator;
import io.netty.handler.codec.http.DefaultFullHttpResponse;
import io.netty.handler.codec.http.FullHttpRequest;
import io.netty.handler.codec.http.FullHttpResponse;
import io.netty.handler.codec.http.HttpResponseStatus;

import java.sql.DriverManager;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Observer;
import java.util.Random;

import javassist.ClassPool;
import javassist.CtClass;
import javassist.bytecode.analysis.FramePrinter;
import net.piraku.concurrent.FutureResource;
import net.piraku.core.CoSync;
import net.piraku.core.PirakuResource;
import net.piraku.core.PirakuResources;
import net.piraku.core.runtime.ClassEnhancer;
import net.piraku.sql.ConnectionResource;
import net.piraku.sql.ResultSetResource;
import net.piraku.sql.StatementResource;

import com.fasterxml.jackson.databind.ObjectMapper;

public class SingleDbQueryHandler {
	
	ConPool pool;
	
	public SingleDbQueryHandler() throws Exception {
		pool = ConPool.getPool();
	}
	
	ConnectionResource getNext() {
		return pool.getNext();
	}
	
	public static class Message {
		private int id;
		private int randomNumber;
		
		public Message(int id, int num) {
			this.id = id;
			this.randomNumber = num;
		}
		
		public int getId() {
			return id;
		}
		public int getRandomNumber() {
			return randomNumber;
		}
		
		@Override
		public String toString() {
			return "id="+id+", num="+randomNumber;
		}
		
	}
	
	ObjectMapper mapper = new ObjectMapper();
	private Random random = new Random();
	
	int count;
	

	@CoSync
	public FullHttpResponse service(FullHttpRequest req) throws Exception {
//		UUID num = UUID.randomUUID();
//		System.out.println("starting "+num);
		ConnectionResource con = getNext();
		
		long nsStart = System.nanoTime();
		StatementResource stmt = con.createStatement();
		FutureResource<ResultSetResource> rsFuture = stmt.executeQueryAsync("select * from world where id = "+random.nextInt(10000));
		
		PirakuResources.await(rsFuture);
		
		long nsStop = System.nanoTime();
		if (count++%10240==0)
			System.out.println("Took: "+(nsStop-nsStart)+" ns for query");

		ResultSetResource rs = rsFuture.get();
		Message msg = null;
		
		if (rs.next())
			msg = new Message(rs.getInt("id"), rs.getInt("randomNumber"));
		
		rs.close();
		stmt.close();
		
		
		byte[] hello = mapper.writeValueAsBytes(msg);
		DefaultFullHttpResponse res = new DefaultFullHttpResponse(req.protocolVersion(), HttpResponseStatus.OK, PooledByteBufAllocator.DEFAULT.buffer(hello.length));
		res.headers().add("Content-Type", "application/json; charset=UTF-8");
		res.headers().add("Server", "Example");
		res.headers().add("Date", new Date());
		res.headers().add("Date", new Date());
		res.headers().add("Content-Length", hello.length);
		if (req.headers().contains("Connection"))
			res.headers().add("Connection", req.headers().get("Connection"));
		else
			res.headers().add("Connection", "Close");
		
		res.content().writeBytes(hello);
		
//		System.out.println("done "+num);
		return res;
	}
	
}
