package net.piraku.core.net;

import io.netty.bootstrap.Bootstrap;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.http.FullHttpRequest;
import io.netty.handler.codec.http.FullHttpResponse;
import io.netty.handler.codec.http.HttpClientCodec;
import io.netty.handler.codec.http.HttpContentDecompressor;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpRequestDecoder;
import io.netty.handler.codec.http.HttpResponseEncoder;

import java.lang.reflect.Method;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.concurrent.locks.AbstractQueuedSynchronizer;

import javassist.ClassPool;
import javassist.CtClass;
import net.piraku.concurrent.FutureResource;
import net.piraku.concurrent.FutureResourceAdapter;
import net.piraku.concurrent.Translator;
import net.piraku.core.Observers;
import net.piraku.core.PirakuResource;
import net.piraku.core.runtime.ClassEnhancer;
import net.piraku.core.runtime.PirakuExec;

public class Http {

	public static final EventLoopGroup ioGroup = new NioEventLoopGroup();
	private int port;

	public Http(int port) {
		this.port = port;
	}

	public void start() throws Exception {
		EventLoopGroup bossGroup = new NioEventLoopGroup(1);
		try {
			ServerBootstrap b = new ServerBootstrap();
			b.group(bossGroup, ioGroup).channel(NioServerSocketChannel.class)
					.childHandler(new MyInititalizer());

			b.bind(port).sync();

		} finally {
//			bossGroup.shutdownGracefully();
		}
	}

	public static FutureResource<FullHttpResponse> request(String host, int port, final FullHttpRequest req) throws Exception {
		MyClientInitit init = new MyClientInitit();
		Bootstrap b = new Bootstrap();
		b.group(ioGroup)
		.channel(NioSocketChannel.class)
		.handler(init);
		
		b.connect(host, port).addListener(new ChannelFutureListener() {
			
			@Override
			public void operationComplete(ChannelFuture future) throws Exception {
				future.channel().writeAndFlush(req);
			}
		});
		
		return init.clientHandler;
	}
	

	private static class MyClientInitit extends ChannelInitializer<SocketChannel> {
		
		MyClientHandler clientHandler = new MyClientHandler();
		
		@Override
		public void initChannel(SocketChannel ch) throws Exception {
			// Create a default pipeline implementation.
			ChannelPipeline p = ch.pipeline();

//			p.addLast("log", new LoggingHandler(LogLevel.INFO));
			// Enable HTTPS if necessary.
			

			p.addLast("codec", new HttpClientCodec());

			// Remove the following line if you don't want automatic content
			// decompression.
			p.addLast("inflater", new HttpContentDecompressor());

			// Uncomment the following line if you don't want to handle
			// HttpChunks.
			 p.addLast("aggregator", new HttpObjectAggregator(1024*1024));

			p.addLast("handler", clientHandler);
		}

	}

	static PirakuExec handler;
	static Method handlerMethod;
	
	public static void main(String[] args) throws Exception {
		Http s = new Http(8090);
		s.start();
		
		Thread.sleep(1000);
		
		ClassPool cp = ClassPool.getDefault();
		ClassEnhancer ce = new ClassEnhancer();
		CtClass handlerClazz = cp.get("net.piraku.core.net.JsonHelloWorld");
		List<CtClass> res = ce.enhance(handlerClazz);
		
		if (res.size()>1)
			res.get(1).toClass();
		
		PirakuExec exec = (PirakuExec) handlerClazz.toClass().newInstance();
		handler = exec;
		for (Method m:handler.getClass().getDeclaredMethods())
			if (m.getName().equals("service"))
				handlerMethod = m;
		
	}

	private static class MyInititalizer extends
			ChannelInitializer<SocketChannel> {
		@Override
		protected void initChannel(SocketChannel ch) throws Exception {
			ChannelPipeline p = ch.pipeline();
			p.addLast("decoder", new HttpRequestDecoder());
			p.addLast("aggregator", new HttpObjectAggregator(1024 * 1024));
			p.addLast("encoder", new HttpResponseEncoder());
			p.addLast("handler", new MyServerHandler());
		}
	}

	public static class MyServerHandler extends SimpleChannelInboundHandler<FullHttpRequest> {

		@Override
		protected void channelRead0(final ChannelHandlerContext ctx, FullHttpRequest msg) throws Exception {
			msg.retain();
			final FutureResource<Object> resFuture = handler.pirakuExecute(handlerMethod, new Object[] { msg });
			resFuture.registerReadyObserver(new PirakuResource.Observer() {
				
				@Override
				public void onReady(PirakuResource resource) {
					try {
						ctx.writeAndFlush(resFuture.get());
					} catch (ExecutionException e) {
						e.printStackTrace();
						ctx.channel().close();
						// create 500 error respone
					} catch (InterruptedException e) {
						e.printStackTrace();
						ctx.channel().close();
						// create 500 error respone
					}
				}
			});
		}
	}

	public static class MyClientHandler extends SimpleChannelInboundHandler<FullHttpResponse> implements FutureResource<FullHttpResponse> {
		
		private FullHttpResponse response;
		private Observers observers = new Observers(this);
		private Sync sync = new Sync();
		
		@Override
		protected void channelRead0(ChannelHandlerContext ctx, FullHttpResponse msg) throws Exception {
			this.response = msg;
			response.retain();
			sync.releaseShared(1);
			observers.fireReadyObservers();
		}
		
		@Override
		public void awaitReady() throws InterruptedException {
			sync.acquireSharedInterruptibly(1);
		}
		
		@Override
		public void awaitReady(long timeout, TimeUnit unit) throws InterruptedException, TimeoutException {
			if (!sync.tryAcquireSharedNanos(1, unit.toNanos(timeout)))
				throw new TimeoutException();
		}
		
		@Override
		public void registerReadyObserver(Observer observer) {
			observers.registerReadyObserver(observer);
			
		}
		
		@Override
		public void removeReadyObserver(Observer observer) {
			observers.removeReadyObserver(observer);
			
		}
		
		private static class Sync extends AbstractQueuedSynchronizer {
			private static final long serialVersionUID = -7932251178455135349L;

			boolean isSignalled() {
				return getState() != 0;
			}

			protected int tryAcquireShared(int ignore) {
				return isSignalled() ? 1 : -1;
			}

			protected boolean tryReleaseShared(int ignore) {
				setState(1);
				return true;
			}
		}
		
		@Override
		public boolean isCancelled() {
			return false;
		}
		@Override
		public boolean cancel(boolean mayInterruptIfRunning) {
			return false;
		}
		
		@Override
		public FullHttpResponse get() throws InterruptedException, ExecutionException {
			
			if (!isReady())
				awaitReady();
			
			return response;
		}
		
		@Override
		public FullHttpResponse get(long timeout, TimeUnit unit) throws InterruptedException, ExecutionException,
				TimeoutException {
			if (!isReady())
				awaitReady(timeout, unit);
			
			return response;
		}
		
		@Override
		public boolean isDone() {
			return isReady();
		}
		
		@Override
		public boolean isReady() {
			return sync.isSignalled();
		}
		
		@Override
		public <O> FutureResource<O> translate(Translator<FullHttpResponse, O> translator) {
			return new FutureResourceAdapter<FullHttpResponse, O>(this, translator);
		}

	}


}
