package org.postgresql;

import java.sql.DriverManager;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import net.piraku.concurrent.FutureResource;
import net.piraku.concurrent.FutureTaskResource;
import net.piraku.core.PirakuResource;
import net.piraku.sql.ConnectionResource;
import net.piraku.sql.PreparedStatementResource;
import net.piraku.sql.ResultSetResource;

public class TestPMt {
	static void log(String s) {
		System.out.println(""+System.nanoTime()+" - "+s);
	}
	
	public static void main(String[] args) throws Exception {
		
		System.in.read();
		
		Class.forName("org.postgresql.Driver");
		
		int num = 1;
		if (args.length > 0)
			num = Integer.parseInt(args[0]);

		int runners = 4;
		if (args.length > 1)
			runners = Integer.parseInt(args[1]);
		
		List<ConnectionResource> cons = new ArrayList<ConnectionResource>();
		List<PirakuResource> lasts = new ArrayList<PirakuResource>();
		
		String url = "jdbc:postgresql://192.168.0.66:5432/test";
		if (null!=System.getProperty("jdbc.url"))
			url = System.getProperty("jdbc.url");
		
		log("start : "+url);
		for (int i=0; i<num; i++) {
			cons.add((ConnectionResource) DriverManager.getConnection(
		   url,"postgres", "postgres"));
			cons.get(i).setAutoCommit(false);
			cons.get(i).setReadOnly(true);
			lasts.add(null);
		}
		
		log("got connections");
		
		long ts0 = System.currentTimeMillis();
		
		int max = 100000;
		if (args.length>2)
			max = Integer.parseInt(args[2]);
		
		ExecutorService es = Executors.newCachedThreadPool();
		
		for (int i=0; i<runners; i++)
			es.execute(new FutureTaskResource<Void>(new Runner(num, (max/runners)*i, (max/runners)*(i+1), cons, lasts), null));
		
		es.shutdown();
		es.awaitTermination(Long.MAX_VALUE, TimeUnit.NANOSECONDS);
		
		long ts1 = System.currentTimeMillis();
		
		
		for (int i=0; i<num; i++)
			cons.get(i).close();
		
		
//		System.out.println("total: "+statements.size());
		System.out.println("closed");
		
		System.out.println("total time: "+(ts1-ts0)+" ms for "+num);
		System.out.println("total "+((1.0*max/(ts1-ts0))*1000)+" ts/sec for "+num);
		
		System.exit(0);
	}
	
	
	static class Runner implements Runnable {
		final int from, to, num;
		List<PirakuResource> lasts;
		List<ConnectionResource> cons;
		PirakuResource last;
		
		Runner(int num, int from, int to, List<ConnectionResource> cons, List<PirakuResource> lasts) {
			this.from = from;
			this.to = to;
			this.cons = cons;
			this.num = num;
			this.lasts = new ArrayList<PirakuResource>(lasts);
		}
		
		@Override
		public void run() {
			try {
				for (int i=from; i<to; i++) {
					lasts.set(i%num, time(cons.get(i%num), i));
				}
			
				for (int i=0; i<num; i++) {
					lasts.get(i).awaitReady();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	static PirakuResource time(ConnectionResource connection, final int num) throws Exception {
		final PreparedStatementResource stm = connection.prepareStatement("select id, randomNumber from world where id = ?");
//		final StatementResource stm = connection.createStatement();
//		log("got stm");
		stm.setInt(1, num%10000);
//		if (num==10000)
//			System.out.println("gothas");
//		final FutureResource<ResultSetResource> rsFuture = stm.executeQueryAsync( "+ (num%10000));
		final FutureResource<ResultSetResource> rsFuture = stm.executeQueryAsync();
//		log("got rs");
		
		rsFuture.registerReadyObserver(new PirakuResource.Observer() {
			
			@Override
			public void onReady(PirakuResource resource) {
				if (num%1024==0)
					log("done - "+num);
				resource.removeReadyObserver(this);
				try {
					ResultSetResource rs = rsFuture.get();
					if (rs.next()) {
						int rc = rs.getInt(2);
						if (num%10000 != rc)
							System.out.println("ERROR - "+num+" wrong rc: "+rc);;
					} else
						System.out.println("ERROR - "+num+" no results");
					rs.close();
					stm.close();
				} catch (Exception e) {
//					log("ERROR - "+num + " cause:"+ e.getMessage());
//					e.printStackTrace(System.out);
				} finally {
//					statements.add(stm);
				}
			}
		});
		
		return rsFuture;
		
	}
}
