package org.postgresql.core;

import java.sql.SQLException;
import java.util.Collections;
import java.util.LinkedList;
import java.util.concurrent.Callable;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;

import net.piraku.concurrent.FutureResource;
import net.piraku.concurrent.FutureTaskResource;
import net.piraku.core.PirakuResource;
import net.piraku.core.PirakuResource.Observer;

import org.postgresql.copy.CopyOperation;

public class QueryExecutorAsync implements QueryExecutor {
	private final QueryExecutorLegacy legacy;
	private static final Executor EXEC = Executors.newCachedThreadPool();
	private AtomicInteger counter = new AtomicInteger();
	private java.util.List<FutureTaskResource<?>> tasks = Collections.synchronizedList(new LinkedList<FutureTaskResource<?>>());
	
	public QueryExecutorAsync(QueryExecutorLegacy leg) {
		legacy = leg;
	}
	
	@Override
	public ParameterList createFastpathParameters(int count) {
		return legacy.createFastpathParameters(count);
	}
	
	@Override
	public Query createParameterizedQuery(String sql) {
		return legacy.createParameterizedQuery(sql);
	}
	
	@Override
	public Query createSimpleQuery(String sql) {
		return legacy.createSimpleQuery(sql);
	}
	
	@Override
	public FutureResource<Void> executeAsync(final Query query, final ParameterList parameters,
			final ResultHandler handler, final int maxRows, final int fetchSize, final int flags)
			throws SQLException {
		
		FutureTaskResource<Void> task = new FutureTaskResource<Void>(new Callable<Void>() {
			@Override
			public Void call() throws Exception {
				legacy.execute(query, parameters, handler, maxRows, fetchSize, flags);
				return null;
			}
		});
		
		return enqueue(task, false);
	}
	@Override
	public FutureResource<Void> executeAsync(final Query[] queries,
			final ParameterList[] parameterLists, final ResultHandler handler, final int maxRows,
			final int fetchSize, final int flags) throws SQLException {
		
		FutureTaskResource<Void> task = new FutureTaskResource<Void>(new Callable<Void>() {
			@Override
			public Void call() throws Exception {
				legacy.execute(queries, parameterLists, handler, maxRows, fetchSize, flags);
				return null;
			}
		});
		
		
		return enqueue(task, false);
	}
	@Override
	public FutureResource<byte[]> fastpathCallAsync(final int fnid, final ParameterList params,
			final boolean suppressBegin) throws SQLException {
		
		FutureTaskResource<byte[]> task = new FutureTaskResource<byte[]>(new Callable<byte[]>() {
			@Override
			public byte[] call() throws Exception {
				return legacy.fastpathCall(fnid, params, suppressBegin);
			}
		});
		
		
		return enqueue(task, true);
	}
	@Override
	public FutureResource<Void> fetchAsync(final ResultCursor cursor,
			final ResultHandler handler, final int fetchSize) throws SQLException {
		FutureTaskResource<Void> task = new FutureTaskResource<Void>(new Callable<Void>() {
			@Override
			public Void call() throws Exception {
				legacy.fetch(cursor, handler, fetchSize);
				return null;
			}
		});
		
		
		return enqueue(task, true);
	}
	
	private Observer taskDoneObserver = new Observer() {
		
		public void onReady(PirakuResource resource) {
			resource.removeReadyObserver(this);
			int num = counter.decrementAndGet();
			if (num>0)
				EXEC.execute(tasks.remove(0));
		}
	};
	
	private <V> FutureTaskResource<V> enqueue(FutureTaskResource<V> task, boolean top) {
		
		task.registerReadyObserver(taskDoneObserver);		
		int num = counter.incrementAndGet();
		
		if (top)
			tasks.add(0, task);
		else
			tasks.add(task);
		
		if (num==1) {
			EXEC.execute(tasks.remove(0));
		}
		
		return task;
	}
	
	@Override
	public void processNotifies() throws SQLException {
		legacy.processNotifies();
	}
	@Override
	public CopyOperation startCopy(String sql, boolean suppressBegin)
			throws SQLException {
		throw new NoSuchMethodError();
	}
	
}
