package org.postgresql.core.netty;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufAllocator;

import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import net.piraku.core.Observers;
import net.piraku.core.PirakuResource;

public class ByteBufInputStream extends InputStream implements PirakuResource {
	private ByteBuf current = null;
//	private AtomicInteger counter = new AtomicInteger();
	private BlockingQueue<ByteBuf> queue = new LinkedBlockingQueue<ByteBuf>();
	private volatile boolean closed;
	private Observers observers = new Observers(this);
	
	static void log(String msg) {
		System.out.println("" + Thread.currentThread()+ " " +System.nanoTime()+" "+msg);
	}
	
	private ByteBuf getCurrent() {
//		ByteBuf current = this.current;
		if (null!=current && current.isReadable())
			return current;
		
		if (null!=current) {
			current.release();
			current = null;
		}
		
		try {
			while (true) {
				
				if (closed && queue.isEmpty())
					return null;
				
//				log("waiting");
				
//				new Throwable().printStackTrace(System.out);
				current = queue.take();
				
				if (current.isReadable())
					return current;
				
				current.release();
				current = null;
			}
		} catch (InterruptedException e) {
			throw new RuntimeException(e);
		}
	}
	
	@Override
	public int read() throws IOException {
		ByteBuf bb = getCurrent();
		if (null==bb)
			return -1;
		
		return bb.readUnsignedByte();
	}
	
	volatile long count;
	
	@Override
	public int read(byte[] b, int off, int len) throws IOException {
		ByteBuf bb = getCurrent();
		if (null==bb)
			return -1;
		
		int newlen = Math.min(len, bb.readableBytes());
		bb.readBytes(b, off, newlen);
		count += newlen;
		return newlen;
	}
	
	public void addByteBuf(ByteBuf buf) {
//		log("add "+buf.readableBytes()+ " " + buf);
		queue.add(buf);
		observers.fireReadyObservers();
	}
	
	public void markClosed() {
		closed = true;
		ByteBuf buf  = ByteBufAllocator.DEFAULT.buffer(128);
		buf.retain();
		queue.add(buf);
	}
	
	@Override
	public int available() throws IOException {
		ByteBuf current = getCurrent();
		if (null==current)
			return -1;
		
		return current.readableBytes();
	}
	
	
	@Override
	public void awaitReady() throws InterruptedException {
		throw new NoSuchMethodError();
	}
	
	@Override
	public void awaitReady(long timeout, TimeUnit unit)
			throws InterruptedException, TimeoutException {
		throw new NoSuchMethodError();
	}
	
	@Override
	public boolean isReady() {
		ByteBuf current = this.current;
		return (closed || (null!=current && current.isReadable()) || !queue.isEmpty());
	}
	
	@Override
	public void registerReadyObserver(Observer observer) {
		observers.registerReadyObserver(observer);
	}
	@Override
	public void removeReadyObserver(Observer observer) {
		observers.removeReadyObserver(observer);
		
	}
	
}
