package org.postgresql.core.netty;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufAllocator;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.channel.Channel;

import java.io.IOException;
import java.io.OutputStream;

public class ByteBufOutputStream extends OutputStream {
	public static interface FlushIndicator {
		boolean shouldFlush();
	}
	
	private FlushIndicator flushIndicator;
	private static ByteBufAllocator allocator = PooledByteBufAllocator.DEFAULT;
	private volatile ByteBuf current = null;
	private final Channel channel;
	
	public ByteBufOutputStream(Channel out) {
		channel = out;
	}
	
	@Override
	public void write(int b) throws IOException {
		if (null==current || !current.isWritable()) {
			if (null!=current)
				channel.write(current);
			
			current = allocator.buffer(1024);
		}
		
		current.writeByte(b);
	}
	
	@Override
	public void write(byte[] b, int off, int len) throws IOException {
		if (null==current || !current.isWritable(len)) {
			if (null!=current)
				channel.write(current);
			
			current = allocator.buffer(Math.max(len, 1024));
		}
		
		current.writeBytes(b, off, len);
	}
	
	static void log(String s) {
		System.out.println(""+System.nanoTime()+" - "+s);
	}

	int skipps;
	
	public void flush() throws IOException {
//		log ("flushing");
		
		if (null!=flushIndicator && !flushIndicator.shouldFlush()) {
			skipps++;
			
			if (skipps==1024)
				skipps = 0;
			else
				return;
			
		}
		
		if (null!=current) {
			channel.write(current);
			current = null;
		}
		
		channel.flush();
	};
	
	public void setFlushIndicator(FlushIndicator flushIndicator) {
		this.flushIndicator = flushIndicator;
	}
}
