package org.postgresql.core.netty;

import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.channel.AdaptiveRecvByteBufAllocator;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.DefaultMessageSizeEstimator;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;

import java.io.IOException;

import net.piraku.core.net.Http;

import org.postgresql.util.HostSpec;

public class Connector {
	public static final EventLoopGroup ioGroup = Http.ioGroup;
	
	private final ByteBufInputStream in;
	private final ByteBufOutputStream out;
	private final SocketChannel channel;
	
	
	public Connector(HostSpec spec) throws IOException {
		try {
			in = new ByteBufInputStream();
			final ChannelInboundHandler handler = new ChannelInboundHandler(in);
			
			Bootstrap b = new Bootstrap();
			b.group(ioGroup);
			b.option(ChannelOption.ALLOCATOR, PooledByteBufAllocator.DEFAULT);
			b.option(ChannelOption.RCVBUF_ALLOCATOR, new AdaptiveRecvByteBufAllocator(14, 1024, 1024*1024));
			b.option(ChannelOption.ALLOCATOR, PooledByteBufAllocator.DEFAULT);
//			 b.option(ChannelOption.MESSAGE_SIZE_ESTIMATOR, MessageSizeEstimator)
			   //setting buffer size can improve I/O
			   b.option(ChannelOption.SO_SNDBUF, 1048576);
			   b.option(ChannelOption.SO_RCVBUF, 1048576); 

			   // better to have an receive buffer predictor 
//			   b.setOption("receiveBufferSizePredictorFactory", new AdaptiveReceiveBufferSizePredictorFactory(MIN_PACKET_SIZE, INITIAL_PACKET_SIZE, MAX_PACKET_SIZE))  

			   //if the server is sending 1000 messages per sec, optimum write buffer water marks will
			   //prevent unnecessary throttling, Check NioSocketChannelConfig doc   
			   b.option(ChannelOption.WRITE_BUFFER_HIGH_WATER_MARK, 1024* 1024 * 1024);
			   b.option(ChannelOption.WRITE_BUFFER_LOW_WATER_MARK, 1024*1024);
			   b.option(ChannelOption.MESSAGE_SIZE_ESTIMATOR, new DefaultMessageSizeEstimator(1024))
			.channel(NioSocketChannel.class)
			.handler(new ChannelInitializer<SocketChannel>() {
				@Override
				protected void initChannel(SocketChannel ch) throws Exception {
					ChannelPipeline p = ch.pipeline();
					p.addLast("codec", handler);
				}
			});
			
			channel = (SocketChannel) b.connect(spec.getHost(), spec.getPort()).sync().channel();
			channel.config().setTcpNoDelay(true);
			out = new ByteBufOutputStream(channel);
		} catch (InterruptedException e) {
			throw new IOException("Error connecting to server.", e);
		}
	}
	
	public ByteBufInputStream getIn() {
		return in;
	}
	
	public ByteBufOutputStream getOut() {
		return out;
	}
	
	public SocketChannel getChannel() {
		return channel;
	}
	
	public void close() {
		if (channel.isRegistered())
			channel.deregister();
		channel.disconnect();
		channel.close();
	}
}
